@@ -30,13 +30,13 @@
 # THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 
-""" Apply JSON-Patches (according to draft 08) """
+from __future__ import unicode_literals
 
-# http://tools.ietf.org/html/draft-ietf-appsawg-json-patch-05
+""" Apply JSON-Patches (RFC 6902) """
 
 # Will be parsed by setup.py to determine package metadata
 __author__ = 'Stefan Kögl <stefan@skoegl.net>'
-__version__ = '0.11'
+__version__ = '1.1'
 __website__ = 'https://github.com/stefankoegl/python-json-patch'
 __license__ = 'Modified BSD License'
 
@@ -89,10 +89,10 @@ def apply_patch(doc, patch, in_place=False):
     >>> other = apply_patch(doc, [{'op': 'add', 'path': '/baz', 'value': 'qux'}])
     >>> doc is not other
     True
-    >>> other
-    {'foo': 'bar', 'baz': 'qux'}
-    >>> apply_patch(doc, [{'op': 'add', 'path': '/baz', 'value': 'qux'}], in_place=True)
-    {'foo': 'bar', 'baz': 'qux'}
+    >>> other == {'foo': 'bar', 'baz': 'qux'}
+    True
+    >>> apply_patch(doc, [{'op': 'add', 'path': '/baz', 'value': 'qux'}], in_place=True) == {'foo': 'bar', 'baz': 'qux'}
+    True
     >>> doc == other
     True
     """
@@ -300,7 +300,7 @@ class JsonPatch(object):
 
         for operation in self.patch:
             operation = self._get_operation(operation)
-            operation.apply(obj)
+            obj = operation.apply(obj)
 
         return obj
 
@@ -351,6 +351,8 @@ class RemoveOperation(PatchOperation):
         except KeyError as ex:
             raise JsonPatchConflict(str(ex))
 
+        return obj
+
 
 class AddOperation(PatchOperation):
     """Adds an object property or an array element."""
@@ -371,12 +373,19 @@ class AddOperation(PatchOperation):
                 subobj.insert(part, value)
 
         elif isinstance(subobj, dict):
-            subobj[part] = value
+            if part is None:
+                # we're replacing the root
+                obj = value
+
+            else:
+                subobj[part] = value
 
         else:
             raise JsonPatchConflict("can't add to type '%s'"
                                     "" % subobj.__class__.__name__)
 
+        return obj
+
 
 class ReplaceOperation(PatchOperation):
     """Replaces an object property or an array element by new value."""
@@ -385,6 +394,9 @@ class ReplaceOperation(PatchOperation):
         value = self.operation["value"]
         subobj, part = self.pointer.to_last(obj)
 
+        if part is None:
+            return value
+
         if isinstance(subobj, list):
             if part > len(subobj) or part < 0:
                 raise JsonPatchConflict("can't replace outside of list")
@@ -399,6 +411,7 @@ class ReplaceOperation(PatchOperation):
                                     "" % subobj.__class__.__name__)
 
         subobj[part] = value
+        return obj
 
 
 class MoveOperation(PatchOperation):
@@ -412,8 +425,9 @@ class MoveOperation(PatchOperation):
         if from_ptr.contains(self.pointer):
             raise JsonPatchException('Cannot move values into its own children')
 
-        RemoveOperation({'op': 'remove', 'path': self.operation['from']}).apply(obj)
-        AddOperation({'op': 'add', 'path': self.location, 'value': value}).apply(obj)
+        obj = RemoveOperation({'op': 'remove', 'path': self.operation['from']}).apply(obj)
+        obj = AddOperation({'op': 'add', 'path': self.location, 'value': value}).apply(obj)
+        return obj
 
 
 class TestOperation(PatchOperation):
@@ -435,6 +449,8 @@ class TestOperation(PatchOperation):
             if val != value:
                 raise JsonPatchTestFailed('%s is not equal to tested value %s (types %s and %s)' % (val, value, type(val), type(value)))
 
+        return obj
+
 
 class CopyOperation(PatchOperation):
     """ Copies an object property or an array element to a new location """
@@ -443,4 +459,5 @@ class CopyOperation(PatchOperation):
         from_ptr = jsonpointer.JsonPointer(self.operation['from'])
         subobj, part = from_ptr.to_last(obj)
         value = copy.deepcopy(subobj[part])
-        AddOperation({'op': 'add', 'path': self.location, 'value': value}).apply(obj)
+        obj = AddOperation({'op': 'add', 'path': self.location, 'value': value}).apply(obj)
+        return obj
